package com.hzya.frame.seeyon.service.impl;

import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.hzya.frame.seeyon.entity.CfsLogEntity;
import com.hzya.frame.seeyon.service.ICfsLogService;
import com.hzya.frame.seeyon.service.ISeeYonInterFace;
import com.hzya.frame.web.entity.JsonResultEntity;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Lazy;
import org.springframework.stereotype.Service;

import javax.annotation.PostConstruct;

/**
 * @Description oa单据推送交行日志
 * @Author xiangerlin
 * @Date 2024/3/18 13:40
 **/
@Service
public class CfsLogServiceImpl implements ICfsLogService {


    @Autowired
    protected ISeeYonInterFace seeYonInterFace;

    @PostConstruct
    public void init() {
        seeYonInterFace.setCfslogService(this);
    }
    /**
     * 保存日志
     *
     * @param entity
     * @return
     */
    @Override
    public JsonResultEntity importCfsLog(CfsLogEntity entity) {
        JSONArray jsonArray = new JSONArray();
        JSONObject data = new JSONObject();
        data.put("tab_name_ch",entity.getTab_name_ch());
        data.put("tab_name_en",entity.getTab_name_en());
        data.put("bill_code",getCode(entity));
        data.put("form_app_id",entity.getOaWorkflowEventDataEntity().getFormApp());
        data.put("event_type","onProcessFinished");
        data.put("id",entity.getOaWorkflowEventDataEntity().getId());
        data.put("title","");//流程标题，这个字段暂时取不到
        data.put("result",entity.getResult());//返回的报文
        if (StrUtil.isNotEmpty(entity.getResult())){
            JSONObject resultObj = JSONObject.parseObject(entity.getResult());
            String statusMsg = resultObj.getString("statusMsg");//交行返回的状态
            String serialNo = resultObj.getString("serialNo");//交行返回的流水号
            data.put("status_msg",statusMsg);
            data.put("serial_no",serialNo);
        }
        jsonArray.add(data);
        JSONObject jsonStr = new JSONObject();
        jsonStr.put("type","archives");
        jsonStr.put("templateCode","formmain_0527");
        jsonStr.put("attribute",jsonArray);
        //1、解析数据
        //2、保存日志
        JSONObject jsonObject = new JSONObject();
        jsonObject.put("jsonStr",jsonStr);
        JsonResultEntity jsonResultEntity = seeYonInterFace.thirdInterfaceSend(jsonObject);
        return jsonResultEntity;
    }

    private String getCode(CfsLogEntity entity){
        if (null != entity && null != entity.getOaWorkflowEventDataEntity()){
            String tableName = entity.getTab_name_en();
            String jsonStr = entity.getOaWorkflowEventDataEntity().getBusinessDataStr();
            JSONObject jsonObject = JSONObject.parseObject(jsonStr);
            if (null != jsonObject){
                String bill_code=null;
                switch (tableName){
                    case "formmain_0307"://差旅报销单
                        bill_code=jsonObject.getJSONObject("formmain_0307").getString("field0094");
                        break;
                    case "formmain_0294"://招待费报销单
                        bill_code=jsonObject.getJSONObject("formmain_0294").getString("field0053");
                        break;
                    case "formmain_0314"://日常费用报销单
                        bill_code=jsonObject.getJSONObject("formmain_0314").getString("field0220");
                        break;
                    case "formmain_0362"://资金归集
                        bill_code=jsonObject.getJSONObject("formmain_0362").getString("field0047");
                        break;
                    case "formmain_0464"://资金拨付
                        bill_code=jsonObject.getJSONObject("formmain_0464").getString("field0045");
                        break;
                    case "formmain_0467"://同名账户划转
                        bill_code=jsonObject.getJSONObject("formmain_0467").getString("field0045");
                        break;
                    case "formmain_0293"://借款单(借还款)
                        bill_code=jsonObject.getJSONObject("formmain_0293").getString("field0056");
                        break;
                    case "formmain_0327"://对公付款单
                    bill_code = jsonObject.getJSONObject("formmain_0327").getString("field0146");
                }
                return bill_code;
            }
        }
        return null;
    }
}
