package com.hzya.frame.web.quartz;


import com.alibaba.fastjson.JSONObject;
import com.baomidou.dynamic.datasource.annotation.DSTransactional;
import com.hzya.frame.base.PluginBaseEntity;
import com.hzya.frame.sys.apireflection.service.impl.ApiHandleServiceImpl;

import com.hzya.frame.sysnew.application.plugin.entity.SysApplicationPluginEntity;
import com.hzya.frame.sysnew.integtationTask.IIntegrationTaskCacheableService;
import com.hzya.frame.sysnew.integtationTask.entity.IntegrationTaskEntity;
import com.hzya.frame.sysnew.integtationTask.service.impl.IntegrationTaskServiceImpl;
import com.hzya.frame.sysnew.integtationTaskLiving.entity.IntegrationTaskLivingEntity;
import com.hzya.frame.sysnew.integtationTaskLiving.service.impl.IntegrationTaskLivingServiceImpl;
import com.hzya.frame.sysnew.integtationTaskLivingDetails.dao.IIntegrationTaskLivingDetailsDao;
import com.hzya.frame.sysnew.integtationTaskLivingDetails.entity.IntegrationTaskLivingDetailsEntity;
import com.hzya.frame.util.PluginUtils;
import com.hzya.frame.uuid.UUIDUtils;
import com.hzya.frame.web.entity.JsonResultEntity;
import org.quartz.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.annotation.Resource;
import java.text.SimpleDateFormat;
import java.time.Duration;
import java.time.LocalDateTime;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * 定时任务运行工厂类
 */
@DisallowConcurrentExecution
public class QuartzJobFactory implements Job {
    private static final Logger logger = LoggerFactory.getLogger(QuartzJobFactory.class);
    @Resource
    private IIntegrationTaskCacheableService iIntegrationTaskCacheableService;
    @Resource
    private IIntegrationTaskLivingDetailsDao integrationTaskLivingDetailsDao;
    @Override
    public void execute(JobExecutionContext jobExecutionContext) throws JobExecutionException {
        //当前开始时间
        Date startTime = new Date();
        Date oldStartTime = new Date();

        logger.info("执行定时任务");
        /** 获取JOB明细对象*/
        JobDetail jobDetail = jobExecutionContext.getJobDetail();
        /** 获取JobMap对象，用来获取实体类*/
        Map<String, Object> obDataMap = jobDetail.getJobDataMap();
        //获取job数据
        JSONObject jsonObjectMap = new JSONObject(obDataMap);
        JSONObject jobDetails = JSONObject.parseObject(jsonObjectMap.getString("job"));
        //任务id
        String integrationTaskId = jobDetails.getString("integrationTaskId");
        IntegrationTaskEntity integrationTaskEntity = iIntegrationTaskCacheableService.getIntegrationTaskEntity(integrationTaskId);
        //实例id
        String integrationTaskLivingId = jobDetails.getString("integrationTaskLivingId");
        IntegrationTaskLivingEntity integrationTaskLivingEntity = iIntegrationTaskCacheableService.getIntegrationTaskLivingEntity(integrationTaskLivingId);

        if(integrationTaskEntity == null || integrationTaskEntity.getId() == null || "".equals(integrationTaskEntity.getId())){
            logger.error("当前任务不存在，请联系管理员");
            return;
        }
        if(integrationTaskLivingEntity == null || integrationTaskLivingEntity.getId() == null || "".equals(integrationTaskLivingEntity.getId())){
            logger.error("当前任务实例不存在，请联系管理员");
            return;
        }
        if(integrationTaskLivingEntity.getOldStartTime() != null ){
            oldStartTime = integrationTaskLivingEntity.getOldStartTime();
        }
        //判断任务是否启用，存在不启用情况，需要手动把任务关闭
        if(integrationTaskEntity.getTaskStatus() != null && !"1".equals(integrationTaskEntity.getTaskStatus())){
            saveLivingAndDetail(integrationTaskLivingEntity,null,
                    "4","4","当前任务已停止，不执行任务",integrationTaskLivingEntity.getOldStartTime(),integrationTaskLivingEntity.getStartTime(),integrationTaskLivingEntity.getEndTime());
            JSONObject stopJson = new JSONObject();
            stopJson.put("jobName",integrationTaskEntity.getTaskName());
            stopJson.put("jobCode",integrationTaskEntity.getTaskCode());
            QuartzJobFactoryUtils.pauseTrigger(stopJson);
            return;
        }
        //任务日志
        IntegrationTaskLivingDetailsEntity integrationTaskLivingDetailsEntity = new IntegrationTaskLivingDetailsEntity();
        integrationTaskLivingDetailsEntity.setId(UUIDUtils.getUUID());
        integrationTaskLivingDetailsEntity.setTaskLinvingId(integrationTaskLivingEntity.getId());
        integrationTaskLivingDetailsEntity.setTaskId(integrationTaskEntity.getId());
        integrationTaskLivingDetailsEntity.setTaskCode(integrationTaskEntity.getTaskCode());
        integrationTaskLivingDetailsEntity.setTaskName(integrationTaskEntity.getTaskName());
        integrationTaskLivingDetailsEntity.setTaskStatus("1");
        integrationTaskLivingDetailsEntity.setStartTime(startTime);//当前任务开始时间
        integrationTaskLivingDetailsEntity.setCompanyId(integrationTaskEntity.getCompanyId());
        integrationTaskLivingDetailsEntity.setOrg_id(integrationTaskEntity.getOrg_id());
        integrationTaskLivingDetailsEntity.setCreate_user_id(integrationTaskEntity.getCreate_user_id());
        integrationTaskLivingDetailsEntity.setModify_user_id(integrationTaskEntity.getModify_user_id());
        integrationTaskLivingDetailsEntity.setCreate_time(new Date());
        integrationTaskLivingDetailsEntity.setModify_time(new Date());
        integrationTaskLivingDetailsEntity.setSts("Y");
        integrationTaskLivingDetailsEntity.setRemark("任务执行中，请等待");
        integrationTaskLivingDetailsDao.save(integrationTaskLivingDetailsEntity);
        integrationTaskLivingEntity.setTaskStatus("1");//执行中
        integrationTaskLivingEntity.setStartTime(startTime);
        integrationTaskLivingEntity.setModify_time(new Date());
        iIntegrationTaskCacheableService.updateIntegrationTaskLivingEntity(integrationTaskLivingEntity);
        //判断插件是否启用
        SysApplicationPluginEntity sysApplicationPluginEntity = iIntegrationTaskCacheableService.getSysApplicationPluginEntity(integrationTaskEntity.getTaskPlugin());
        if(sysApplicationPluginEntity == null || sysApplicationPluginEntity.getPluginCode() == null || !"1".equals(sysApplicationPluginEntity.getPluginStatus())){
            saveLivingAndDetail(integrationTaskLivingEntity,integrationTaskLivingDetailsEntity,
                    "2","3","插件未启用，不执行任务",integrationTaskLivingEntity.getOldStartTime(),startTime,new Date());
            return;
        }
        //查找插件
        PluginBaseEntity pluginBaseEntity = PluginUtils.getPluginsById(sysApplicationPluginEntity.getPluginCode());
        if(pluginBaseEntity == null ){
            saveLivingAndDetail(integrationTaskLivingEntity,integrationTaskLivingDetailsEntity,
                    "2","3","插件未找到，不执行任务",integrationTaskLivingEntity.getOldStartTime(),startTime,new Date());
            return;
        }
        //调用插件
        JSONObject req = new JSONObject();
        req.put("startTime",oldStartTime);
        req.put("endTime",startTime);
        try {
            JsonResultEntity jsonResultEntity = pluginBaseEntity.executeBusiness(req);
            if(jsonResultEntity.isFlag()){
                //写日志，修改任务实例
                saveLivingAndDetail(integrationTaskLivingEntity,integrationTaskLivingDetailsEntity,
                        "2","2",jsonResultEntity.getMsg(),startTime,startTime,new Date());
            }else {
                //写日志，修改任务实例
                saveLivingAndDetail(integrationTaskLivingEntity,integrationTaskLivingDetailsEntity,
                        "3","3",jsonResultEntity.getMsg(),startTime,startTime,new Date());
            }
        } catch (Exception e) {
            saveLivingAndDetail(integrationTaskLivingEntity,integrationTaskLivingDetailsEntity,
                    "3","3","插件执行失败，请联系管理员",startTime,startTime,new Date());
            logger.error("插件执行失败:"+e.getMessage());
        }
    }

    /**
     * @Author lvleigang
     * @Description   保存任务日志
     * @Date 3:07 下午 2024/5/7
     * @param integrationTaskLivingEntity 任务实例
     * @param livingtype 1、执行中2、待执行3、错误 4、当前任务已停止
     * @param detailtype 1、执行中2、执行成功 3、执行失败 4\当前任务已停止
     * @param msg
     * @return void
     **/
    private void saveLivingAndDetail(IntegrationTaskLivingEntity integrationTaskLivingEntity,
                                     IntegrationTaskLivingDetailsEntity integrationTaskLivingDetailsEntity,
                                     String livingtype,String detailtype, String msg,Date oldStartTime,Date startTime,Date endTime) {
        Long interval = (endTime.getTime() - startTime.getTime()) / (1000); // 计算间隔天数
        integrationTaskLivingEntity.setTaskStatus(livingtype);//待执行
        integrationTaskLivingEntity.setOldStartTime(oldStartTime);
        integrationTaskLivingEntity.setStartTime(startTime);
        integrationTaskLivingEntity.setEndTime(endTime);
        integrationTaskLivingEntity.setRunNum(interval.intValue());
        integrationTaskLivingEntity.setRunNum(integrationTaskLivingEntity.getRunNum()+1);
        integrationTaskLivingEntity.setModify_time(new Date());
        iIntegrationTaskCacheableService.updateIntegrationTaskLivingEntity(integrationTaskLivingEntity);
        if(integrationTaskLivingDetailsEntity != null ){
            integrationTaskLivingDetailsEntity.setEndTime(endTime);
            integrationTaskLivingDetailsEntity.setDiffTime(interval.toString());
            integrationTaskLivingDetailsEntity.setTaskStatus(detailtype);//执行成功
            integrationTaskLivingDetailsEntity.setRemark(msg);
            integrationTaskLivingDetailsDao.update(integrationTaskLivingDetailsEntity);
        }
    }
}
